cmake_minimum_required(VERSION 3.20...3.26)
function(useCompilerCache)
    message(NOTICE "CMAKE_CURRENT_SOURCE_DIR ${CMAKE_CURRENT_SOURCE_DIR} and CMAKE_SOURCE_DIR ${CMAKE_SOURCE_DIR}")
  if(NOT CMAKE_CURRENT_SOURCE_DIR STREQUAL CMAKE_SOURCE_DIR)
      message(NOTICE "Here ")
    return()
  endif()
  if ($ENV{TWOSWindows} STREQUAL "1")
  message(NOTICE "PATH $ENV{TWToolsHome}/ccache")
    find_program(CCACHE_EXECUTABLE ccache HINTS "$ENV{TWToolsHome}/ccache" PATHS "$ENV{TWToolsHome}/ccache")
  else ()
    find_program(CCACHE_EXECUTABLE ccache)
  endif ()
  set(CCACHE_PROGRAM $ENV{TWToolsHome}/ccache/ccache.exe)
  message(NOTICE "CCACHE_EXECUTABLE ${CCACHE_EXECUTABLE}")
  if(NOT CCACHE_EXECUTABLE)
    return()
  endif()
  # Ccache 4.7.4 manual says -fno-pch-timestamp is required for Clang
  foreach(lang IN ITEMS C CXX OBJC OBJCXX)
    if(CMAKE_${lang}_COMPILER_ID MATCHES "Clang")
        add_compile_options(
        "$<$<COMPILE_LANGUAGE:${lang}>:SHELL:-Xclang -fno-pch-timestamp>"
        )
    endif()
  endforeach()
  message(NOTICE "MSVC ${MSVC}")
  if(MSVC)
      # Disable use of separate PDB, Ccache won't cache things otherwise
      foreach(lang IN ITEMS CXX)
          message(NOTICE "CMAKE_CONFIGURATION_TYPES ${CMAKE_CONFIGURATION_TYPES}")
          foreach(config IN LISTS CMAKE_BUILD_TYPE CMAKE_CONFIGURATION_TYPES)
              set(var CMAKE_${lang}_FLAGS)
              if(NOT config STREQUAL "")
                  string(TOUPPER "${config}" config)
                  string(APPEND var "_${config}")
                  message(NOTICE "HERE VAR is ${var}")
              endif()
               message(NOTICE "useCompilerCache1-> ${var} ->${${var}}")

              string(REGEX REPLACE "[-/]Z[iI]" "-Z7" ${var} "${${var}}")
               set(${var} "${${var}}" PARENT_SCOPE)
               message(NOTICE "useCompilerCache ${${var}}")
          endforeach()
      endforeach()
      
     if(DEFINED CMAKE_MSVC_DEBUG_INFORMATION_FORMAT)
         string(REGEX REPLACE "ProgramDatabase|EditAndContinue" "Embedded"
         replaced "${CMAKE_MSVC_DEBUG_INFORMATION_FORMAT}"
         )
         set(CMAKE_MSVC_DEBUG_INFORMATION_FORMAT "${replaced}" PARENT_SCOPE)
     else()
         set(CMAKE_MSVC_DEBUG_INFORMATION_FORMAT
         "$<$<CONFIG:Debug,RelWithDebInfo>:Embedded>"
         PARENT_SCOPE
         )
     endif()
  endif()

  # Use a cache variable so the user can override this
  #,include_file_mtime,include_file_ctime
  set(CCACHE_ENV CCACHE_SLOPPINESS=pch_defines,time_macros
  CACHE STRING "List of environment variables for ccache, each in key=value form"
  )
  if(CMAKE_GENERATOR MATCHES "Ninja|Makefiles")
    foreach(lang IN ITEMS C CXX OBJC OBJCXX CUDA)
        set(CMAKE_${lang}_COMPILER_LAUNCHER
        ${CMAKE_COMMAND} -E env ${CCACHE_ENV} ${CCACHE_EXECUTABLE}
        PARENT_SCOPE
        )
    endforeach()
  elseif(CMAKE_GENERATOR STREQUAL Xcode)
    foreach(lang IN ITEMS C CXX)
        list(JOIN CCACHE_ENV "\nexport " setEnv)
        if(NOT setEnv STREQUAL "")
            string(PREPEND setEnv "export ")
        endif()
        set(launch${lang} ${CMAKE_BINARY_DIR}/launch-${lang})
        file(WRITE ${launch${lang}}
        "#!/bin/bash\n"
        "${setEnv}\n"
        "exec \"${CCACHE_EXECUTABLE}\" \"${CMAKE_${lang}_COMPILER}\" \"$@\"\n"
        )
        execute_process(COMMAND chmod a+rx ${launch${lang}})
    endforeach()
    set(CMAKE_XCODE_ATTRIBUTE_CC ${launchC} PARENT_SCOPE)
    set(CMAKE_XCODE_ATTRIBUTE_CXX ${launchCXX} PARENT_SCOPE)
    set(CMAKE_XCODE_ATTRIBUTE_LD ${launchC} PARENT_SCOPE)
    set(CMAKE_XCODE_ATTRIBUTE_LDPLUSPLUS ${launchCXX} PARENT_SCOPE)
  elseif(CMAKE_GENERATOR MATCHES "Visual Studio")
      message(NOTICE "I am HERE")
    cmake_path(NATIVE_PATH CCACHE_EXECUTABLE ccacheExe)
    list(JOIN CCACHE_ENV "\nset " setEnv)
    if(NOT setEnv STREQUAL "")
        string(PREPEND setEnv "set ")
    endif()
    # At least one of C or CXX must be enabled
    get_property(langs GLOBAL PROPERTY ENABLED_LANGUAGES)
    message(NOTICE "ENABLED_LANGUAGES ${langs} ")
    if(CXX IN_LIST langs)
        set(compiler "${CMAKE_CXX_COMPILER}")
    else()
        set(compiler "${CMAKE_C_COMPILER}")
    endif()
    file(WRITE ${CMAKE_BINARY_DIR}/launch-cl.cmd
    "@echo off\n"
    "${setEnv}\n"
    "\"${ccacheExe}\" \"${compiler}\" %*\n"
    )

    file(READ ${CMAKE_BINARY_DIR}/launch-cl.cmd cmdstr )
    message(NOTICE "cmdstr ${cmdstr}")
    list(FILTER CMAKE_VS_GLOBALS EXCLUDE REGEX "^(CLTool(Path|Exe)|TrackFileAccess)=.*$")
    list(APPEND CMAKE_VS_GLOBALS
    CLToolPath=${CMAKE_BINARY_DIR}
    CLToolExe=launch-cl.cmd
    TrackFileAccess=false
    )
    if(NOT CMAKE_VS_GLOBALS MATCHES "(^|;)UseMultiToolTask=")
        list(APPEND CMAKE_VS_GLOBALS UseMultiToolTask=true)
    endif()
    if(NOT CMAKE_VS_GLOBALS MATCHES "(^|;)EnforceProcessCountAcrossBuilds=")
        list(APPEND CMAKE_VS_GLOBALS EnforceProcessCountAcrossBuilds=true)
    endif()
    set(CMAKE_VS_GLOBALS "${CMAKE_VS_GLOBALS}" PARENT_SCOPE)
    message(NOTICE "CMAKE_VS_GLOBALS ${CMAKE_VS_GLOBALS}")
    endif()
endfunction()
